@@ -1,28 +1,24 @@
 """logging"""
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2011 Brian Granger, Min Ragan-Kelley
-#
-#  This file is part of pyzmq, copied and adapted from h5py.
-#  h5py source used under the New BSD license
-#
-#  h5py: <http://code.google.com/p/h5py/>
-#
-#  Distributed under the terms of the New BSD License.  The full license is in
-#  the file COPYING.BSD, distributed as part of this software.
-#-----------------------------------------------------------------------------
+
+# Copyright (c) PyZMQ Developers.
+# Distributed under the terms of the Modified BSD License.
 
 from __future__ import division
 
+import os
 import sys
 import logging
 
 #-----------------------------------------------------------------------------
-# Logging (adapted from h5py: http://h5py.googlecode.com)
+# Logging (adapted from h5py: https://www.h5py.org/)
 #-----------------------------------------------------------------------------
 
 
 logger = logging.getLogger()
-logger.setLevel(logging.INFO)
+if os.environ.get('DEBUG'):
+    logger.setLevel(logging.DEBUG)
+else:
+    logger.setLevel(logging.INFO)
 logger.addHandler(logging.StreamHandler(sys.stderr))
 
 def debug(msg):
