@@ -5,49 +5,47 @@
 import time
 
 import zmq
-from zmq.asyncio import Context, Poller, ZMQEventLoop
+from zmq.asyncio import Context, Poller
 import asyncio
 
 url = 'tcp://127.0.0.1:5555'
 
-loop = ZMQEventLoop()
-asyncio.set_event_loop(loop)
+ctx = Context.instance()
 
-ctx = Context()
 
-@asyncio.coroutine
-def ping():
+async def ping():
     """print dots to indicate idleness"""
     while True:
-        yield from asyncio.sleep(0.5)
+        await asyncio.sleep(0.5)
         print('.')
 
-@asyncio.coroutine
-def receiver():
+
+async def receiver():
     """receive messages with polling"""
     pull = ctx.socket(zmq.PULL)
     pull.connect(url)
     poller = Poller()
     poller.register(pull, zmq.POLLIN)
     while True:
-        events = yield from poller.poll()
+        events = await poller.poll()
         if pull in dict(events):
             print("recving", events)
-            msg = yield from pull.recv_multipart()
+            msg = await pull.recv_multipart()
             print('recvd', msg)
 
-@asyncio.coroutine
-def sender():
+
+async def sender():
     """send a message every second"""
     tic = time.time()
     push = ctx.socket(zmq.PUSH)
     push.bind(url)
     while True:
         print("sending")
-        yield from push.send_multipart([str(time.time() - tic).encode('ascii')])
-        yield from asyncio.sleep(1)
+        await push.send_multipart([str(time.time() - tic).encode('ascii')])
+        await asyncio.sleep(1)
 
-loop.run_until_complete(asyncio.wait([
+
+asyncio.get_event_loop().run_until_complete(asyncio.wait([
     ping(),
     receiver(),
     sender(),
