@@ -10,7 +10,7 @@
 import time
 import zmq
 
-print "Running polling tests for PAIR sockets..."
+print("Running polling tests for PAIR sockets...")
 
 addr = 'tcp://127.0.0.1:5555'
 ctx = zmq.Context()
@@ -53,4 +53,4 @@ poller.unregister(s2)
 # Wait for everything to finish.
 time.sleep(1.0)
 
-print "Finished."
\ No newline at end of file
+print("Finished.")
