@@ -2,6 +2,9 @@
 """Wrapper to run setup.py using setuptools."""
 
 import os, sys
+import warnings
+
+warnings.warn("setupegg.py is deprecated. Don't use it anymore, it's a bit silly.")
 
 # now, import setuptools and call the actual setup
 import setuptools
