@@ -4,9 +4,6 @@
 # Copyright (C) PyZMQ Developers
 # Distributed under the terms of the Modified BSD License.
 
-import random
-import codecs
-
 import errno as errno_mod
 
 from ._cffi import (C, ffi, new_uint64_pointer, new_int64_pointer,
@@ -15,7 +12,8 @@ from ._cffi import (C, ffi, new_uint64_pointer, new_int64_pointer,
                     IPC_PATH_MAX_LEN)
 
 from .message import Frame
-from .constants import *
+from .constants import RCVMORE
+from .utils import _retry_sys_call
 
 import zmq
 from zmq.error import ZMQError, _check_rc, _check_version
@@ -64,10 +62,13 @@ class Socket(object):
     _closed = None
     _ref = None
     _shadow = False
+    copy_threshold = 0
 
     def __init__(self, context=None, socket_type=None, shadow=None):
         self.context = context
         if shadow is not None:
+            if isinstance(shadow, Socket):
+                shadow = shadow.underlying
             self._zmq_socket = ffi.cast("void *", shadow)
             self._shadow = True
         else:
@@ -76,27 +77,46 @@ class Socket(object):
         if self._zmq_socket == ffi.NULL:
             raise ZMQError()
         self._closed = False
-        if context:
-            self._ref = context._add_socket(self)
     
     @property
     def underlying(self):
         """The address of the underlying libzmq socket"""
         return int(ffi.cast('size_t', self._zmq_socket))
-    
+
+    def _check_closed_deep(self):
+        """thorough check of whether the socket has been closed,
+        even if by another entity (e.g. ctx.destroy).
+
+        Only used by the `closed` property.
+
+        returns True if closed, False otherwise
+        """
+        if self._closed:
+            return True
+        try:
+            self.get(zmq.TYPE)
+        except ZMQError as e:
+            if e.errno == zmq.ENOTSOCK:
+                self._closed = True
+                return True
+            else:
+                raise
+        return False
+
     @property
     def closed(self):
-        return self._closed
+        return self._check_closed_deep()
 
     def close(self, linger=None):
         rc = 0
         if not self._closed and hasattr(self, '_zmq_socket'):
             if self._zmq_socket is not None:
+                if linger is not None:
+                    self.set(zmq.LINGER, linger)
                 rc = C.zmq_close(self._zmq_socket)
             self._closed = True
-            if self.context:
-                self.context._rm_socket(self._ref)
-        return rc
+        if rc < 0:
+            _check_rc(rc)
 
     def bind(self, address):
         if isinstance(address, unicode):
@@ -112,6 +132,14 @@ class Socket(object):
                                 'characters (sizeof(sockaddr_un.sun_path)).'
                                 .format(path, IPC_PATH_MAX_LEN))
                 raise ZMQError(C.zmq_errno(), msg=msg)
+            elif C.zmq_errno() == errno_mod.ENOENT:
+                # py3compat: address is bytes, but msg wants str
+                if str is unicode:
+                    address = address.decode('utf-8', 'replace')
+                path = address.split('://', 1)[-1]
+                msg = ('No such file or directory for ipc path "{0}".'.format(
+                       path))
+                raise ZMQError(C.zmq_errno(), msg=msg)
             else:
                 _check_rc(rc)
 
@@ -150,11 +178,11 @@ class Socket(object):
         c_value_pointer = c_data[0]
         c_sizet = c_data[1]
 
-        rc = C.zmq_setsockopt(self._zmq_socket,
-                               option,
-                               ffi.cast('void*', c_value_pointer),
-                               c_sizet)
-        _check_rc(rc)
+        _retry_sys_call(C.zmq_setsockopt,
+                        self._zmq_socket,
+                        option,
+                        ffi.cast('void*', c_value_pointer),
+                        c_sizet)
 
     def get(self, option):
         c_data = new_pointer_from_opt(option, length=255)
@@ -162,11 +190,11 @@ class Socket(object):
         c_value_pointer = c_data[0]
         c_sizet_pointer = c_data[1]
 
-        rc = C.zmq_getsockopt(self._zmq_socket,
-                               option,
-                               c_value_pointer,
-                               c_sizet_pointer)
-        _check_rc(rc)
+        _retry_sys_call(C.zmq_getsockopt,
+                        self._zmq_socket,
+                        option,
+                        c_value_pointer,
+                        c_sizet_pointer)
         
         sz = c_sizet_pointer[0]
         v = value_from_opt_pointer(option, c_value_pointer, sz)
@@ -182,13 +210,17 @@ class Socket(object):
             message = message.bytes
 
         zmq_msg = ffi.new('zmq_msg_t*')
+        if not isinstance(message, bytes):
+            # cast any bufferable data to bytes via memoryview
+            message = memoryview(message).tobytes()
+
         c_message = ffi.new('char[]', message)
         rc = C.zmq_msg_init_size(zmq_msg, len(message))
-        C.memcpy(C.zmq_msg_data(zmq_msg), c_message, len(message))
-
-        rc = C.zmq_msg_send(zmq_msg, self._zmq_socket, flags)
-        C.zmq_msg_close(zmq_msg)
         _check_rc(rc)
+        C.memcpy(C.zmq_msg_data(zmq_msg), c_message, len(message))
+        _retry_sys_call(C.zmq_msg_send, zmq_msg, self._zmq_socket, flags)
+        rc2 = C.zmq_msg_close(zmq_msg)
+        _check_rc(rc2)
 
         if track:
             return zmq.MessageTracker()
@@ -196,16 +228,17 @@ class Socket(object):
     def recv(self, flags=0, copy=True, track=False):
         zmq_msg = ffi.new('zmq_msg_t*')
         C.zmq_msg_init(zmq_msg)
-
-        rc = C.zmq_msg_recv(zmq_msg, self._zmq_socket, flags)
-
-        if rc < 0:
+        
+        try:
+            _retry_sys_call(C.zmq_msg_recv, zmq_msg, self._zmq_socket, flags)
+        except Exception:
             C.zmq_msg_close(zmq_msg)
-            _check_rc(rc)
+            raise
 
         _buffer = ffi.buffer(C.zmq_msg_data(zmq_msg), C.zmq_msg_size(zmq_msg))
         value = _buffer[:]
-        C.zmq_msg_close(zmq_msg)
+        rc = C.zmq_msg_close(zmq_msg)
+        _check_rc(rc)
 
         frame = Frame(value, track=track)
         frame.more = self.getsockopt(RCVMORE)
@@ -232,12 +265,14 @@ class Socket(object):
         events : int [default: zmq.EVENT_ALL]
             The zmq event bitmask for which events will be sent to the monitor.
         """
-        
+
         _check_version((3,2), "monitor")
         if events < 0:
             events = zmq.EVENT_ALL
         if addr is None:
             addr = ffi.NULL
+        if isinstance(addr, unicode):
+            addr = addr.encode('utf8')
         rc = C.zmq_socket_monitor(self._zmq_socket, addr, events)
 
 
